#include <algorithm>
#include <cstdlib>
#include <stdexcept>
#include "jukebox.h"
#include "playlist.h"
#include "sounddecoder.h"

using namespace std;

namespace Msp {
namespace AL {

Jukebox::Jukebox():
	streamer(source),
	in(0),
	decoder(0),
	playlist(0),
	current(0)
{ }

Jukebox::~Jukebox()
{
	streamer.stop();
	delete decoder;
}

void Jukebox::set_playlist(const Playlist *p)
{
	bool was_playing = decoder;
	stop();
	playlist = p;
	if(playlist && was_playing)
		play();
}

void Jukebox::play()
{
	if(!playlist || playlist->empty() || decoder)
		return;

	in = playlist->open(current);
	decoder = SoundDecoder::open_io(*in);
	streamer.play(*decoder);
}

void Jukebox::next()
{
	if(!playlist || playlist->empty())
		return;

	stop();
	current = playlist->advance(current, 1);
	signal_track_changed.emit(current);
	play();
}

void Jukebox::previous()
{
	if(!playlist || playlist->empty())
		return;

	stop();
	current = playlist->advance(current, -1);
	signal_track_changed.emit(current);
	play();
}

void Jukebox::stop()
{
	streamer.stop();
	delete decoder;
	decoder = 0;
	delete in;
	in = 0;
}

void Jukebox::tick()
{
	streamer.tick();
	if(decoder && decoder->eof())
		next();
}

} // namespace AL
} // namespace Msp
