#include "buffer.h"
#include "source.h"

using namespace std;

namespace Msp {
namespace AL {

Source::Source()
{
	alGenSources(1, &id);
}

Source::~Source()
{
	alDeleteSources(1, &id);
}

SourceState Source::get_state() const
{
	int state = INITIAL;
	alGetSourcei(id, AL_SOURCE_STATE, &state);
	return static_cast<SourceState>(state);
}

void Source::set_position(float x, float y, float z)
{
	alSource3f(id, AL_POSITION, x, y, z);
}

void Source::set_velocity(float x, float y, float z)
{
	alSource3f(id, AL_VELOCITY, x, y, z);
}

void Source::set_looping(bool l)
{
	alSourcei(id, AL_LOOPING, l);
}

void Source::set_gain(float g)
{
	alSourcef(id, AL_GAIN, g);
}

void Source::set_rolloff_factor(float f)
{
	alSourcef(id, AL_ROLLOFF_FACTOR, f);
}

void Source::set_buffer(const Buffer &buffer)
{
	alSourcei(id, AL_BUFFER, buffer.get_id());
}

void Source::queue_buffers(const vector<const Buffer *> &buffers)
{
	vector<uint> ids;
	ids.reserve(buffers.size());
	for(vector<const Buffer *>::const_iterator i=buffers.begin(); i!=buffers.end(); ++i)
		ids.push_back((*i)->get_id());
	alSourceQueueBuffers(id, ids.size(), &ids.front());
}

void Source::queue_buffer(const Buffer &buffer)
{
	uint bid = buffer.get_id();
	alSourceQueueBuffers(id, 1, &bid);
}

void Source::unqueue_buffers(const vector<const Buffer *> &buffers)
{
	vector<uint> ids;
	ids.reserve(buffers.size());
	for(vector<const Buffer *>::const_iterator i=buffers.begin(); i!=buffers.end(); ++i)
		ids.push_back((*i)->get_id());
	alSourceUnqueueBuffers(id, ids.size(), &ids.front());
}

void Source::unqueue_buffer(const Buffer &buffer)
{
	uint bid = buffer.get_id();
	alSourceUnqueueBuffers(id, 1, &bid);
}

void Source::clear_buffers()
{
	alSourcei(id, AL_BUFFER, AL_NONE);
}

unsigned Source::get_buffers_queued() const
{
	int n = 0;
	alGetSourceiv(id, AL_BUFFERS_QUEUED, &n);
	return n;
}

unsigned Source::get_buffers_processed() const
{
	int n = 0;
	alGetSourceiv(id, AL_BUFFERS_PROCESSED, &n);
	return n;
}

void Source::play()
{
	alSourcePlay(id);
}

void Source::pause()
{
	alSourcePause(id);
}

void Source::stop()
{
	alSourceStop(id);
}

void Source::rewind()
{
	alSourceRewind(id);
}

} // namespace AL
} // namespace Msp
