#include "buffer.h"
#include "sounddecoder.h"
#include "streamer.h"

using namespace std;

namespace Msp {
namespace AL {

Streamer::Streamer(Source &s):
	src(s),
	decoder(0)
{ }

Streamer::~Streamer()
{
	src.stop();
	src.clear_buffers();
	for(list<Buffer *>::iterator i=buffers.begin(); i!=buffers.end(); ++i)
		delete *i;
}

void Streamer::play(SoundDecoder &s)
{
	decoder = &s;
	tick();
	src.play();
}

void Streamer::stop()
{
	decoder = 0;
	src.stop();
}

void Streamer::tick()
{
	if(unsigned n = src.get_buffers_processed())
	{
		for(unsigned i=0; i<n; ++i)
		{
			src.unqueue_buffer(*buffers.front());
			delete buffers.front();
			buffers.erase(buffers.begin());
		}
	}

	if(!decoder)
		return;

	if(src.get_state()!=PLAYING && src.get_state()!=PAUSED)
		src.play();

	unsigned freq = decoder->get_frequency();
	unsigned chunk_size = freq&~0xF;
	unsigned queued = src.get_buffers_queued();
	vector<char> data(chunk_size);
	for(unsigned i=queued; i<4; ++i)
	{
		unsigned pos = 0;
		while(pos<chunk_size)
		{
			unsigned len = decoder->read(&data[0]+pos, chunk_size-pos);
			if(len==0)
				break;
			pos += len;
		}

		if(pos)
		{
			Buffer *buf = new Buffer;
			buf->data(decoder->get_format(), &data[0], pos, freq);
			src.queue_buffer(*buf);
			buffers.push_back(buf);
		}
	}

	if(decoder->eof())
		decoder = 0;
}

} // namespace AL
} // namespace Msp
